diff --git a/preprocessor/ComputingTasks.cc b/preprocessor/ComputingTasks.cc
index d4ff1708c7c93ee2655d655523f4d030101d7f9c..a3b260fe28c1e63267eaad3d0d55ecfe9892cb53 100644
--- a/preprocessor/ComputingTasks.cc
+++ b/preprocessor/ComputingTasks.cc
@@ -95,6 +95,9 @@ void
 SimulStatement::checkPass(ModFileStructure &mod_file_struct, WarningConsolidation &warnings)
 {
   mod_file_struct.simul_present = true;
+
+  // The following is necessary to allow shocks+endval+simul in a loop
+  mod_file_struct.shocks_present_but_simul_not_yet = false;
 }
 
 void
diff --git a/preprocessor/NumericalInitialization.cc b/preprocessor/NumericalInitialization.cc
index 63d1d73ae0670c290335d4614528feabc98142b1..98327d66383d53cf03f3f821dbeebad8a2d69633 100644
--- a/preprocessor/NumericalInitialization.cc
+++ b/preprocessor/NumericalInitialization.cc
@@ -150,9 +150,9 @@ EndValStatement::EndValStatement(const init_values_t &init_values_arg,
 void
 EndValStatement::checkPass(ModFileStructure &mod_file_struct, WarningConsolidation &warnings)
 {
-  if (mod_file_struct.shocks_present)
+  if (mod_file_struct.shocks_present_but_simul_not_yet)
     {
-      cerr << "ERROR: Putting a \"shocks\" block before an \"endval\" block is not permitted. Please swap the two blocks. This limitation will be removed in the next major release of Dynare." << endl;
+      cerr << "ERROR: Putting a \"shocks\" block before an \"endval\" block is not permitted. Please swap the two blocks. This limitation will be removed in a future release of Dynare." << endl;
       exit(EXIT_FAILURE);
     }
 }
diff --git a/preprocessor/Shocks.cc b/preprocessor/Shocks.cc
index 949ff5f198966b6f633f71d4bad6aadc5d168268..3536053e186c502145b214b1ef687858ddf099e4 100644
--- a/preprocessor/Shocks.cc
+++ b/preprocessor/Shocks.cc
@@ -196,7 +196,7 @@ void
 ShocksStatement::checkPass(ModFileStructure &mod_file_struct, WarningConsolidation &warnings)
 {
   // Workaround for trac ticket #35
-  mod_file_struct.shocks_present = true;
+  mod_file_struct.shocks_present_but_simul_not_yet = true;
 
   // Determine if there is a calibrated measurement error
   for (var_and_std_shocks_t::const_iterator it = var_shocks.begin();
@@ -245,7 +245,7 @@ void
 MShocksStatement::checkPass(ModFileStructure &mod_file_struct, WarningConsolidation &warnings)
 {
   // Workaround for trac ticket #35
-  mod_file_struct.shocks_present = true;
+  mod_file_struct.shocks_present_but_simul_not_yet = true;
 }
 
 ConditionalForecastPathsStatement::ConditionalForecastPathsStatement(const AbstractShocksStatement::det_shocks_t &paths_arg, const SymbolTable &symbol_table_arg) :
diff --git a/preprocessor/Statement.cc b/preprocessor/Statement.cc
index 4c890baffc3b3381b1467b0786a663c085da7634..48abb3a984101f77f422605659606e3dcbf29e1a 100644
--- a/preprocessor/Statement.cc
+++ b/preprocessor/Statement.cc
@@ -37,7 +37,7 @@ ModFileStructure::ModFileStructure() :
   identification_present(false),
   estimation_analytic_derivation(false),
   partial_information(false),
-  shocks_present(false),
+  shocks_present_but_simul_not_yet(false),
   histval_present(false),
   k_order_solver(false),
   calibrated_measurement_errors(false),
diff --git a/preprocessor/Statement.hh b/preprocessor/Statement.hh
index 3847d3cccda706b00eeae9fc7c7f6bbe09b35994..1c5c00829994d27867f410876087c4b30a045944 100644
--- a/preprocessor/Statement.hh
+++ b/preprocessor/Statement.hh
@@ -69,9 +69,11 @@ public:
   bool estimation_analytic_derivation;
   //! Whether the option partial_information is given to stoch_simul/estimation/osr/ramsey_policy
   bool partial_information;
-  //! Whether a shocks or mshocks block is present
-  /*! Used for the workaround for trac ticket #35 */
-  bool shocks_present;
+  //! Whether a shocks or mshocks block has been parsed and no simul command yet run
+  /*! Used for the workaround for trac ticket #35. When a simul command is
+      seen, this flag is cleared in order to allow a sequence
+      shocks+endval+simul in a loop */
+  bool shocks_present_but_simul_not_yet;
   //! Whether a histval bloc is present
   /*! Used for the workaround for trac ticket #157 */
   bool histval_present;